@@ -15,62 +15,14 @@ limitations under the License.
 """
 
 from __future__ import print_function
+
+import datetime
+
+from election_results_xml_validator import loggers
+from election_results_xml_validator import stats
 from lxml import etree
 
 
-class ElectionException(Exception):
-  """Base class for all the errors in this script."""
-  error_message = None
-  description = None
-  error_log = []
-
-  def __init__(self, message):
-    super(ElectionException, self).__init__()
-    self.error_message = message
-
-  def __str__(self):
-    return repr(self.error_message)
-
-
-class ElectionError(ElectionException):
-  """An error that prevents the feed from being processed successfully."""
-
-  description = "Error"
-
-
-class ElectionTreeError(ElectionError):
-  """Special exception for Tree Rules."""
-
-  def __init__(self, message, error_log):
-    super(ElectionTreeError, self).__init__(message)
-    self.error_log = error_log
-
-
-class ElectionWarning(ElectionException):
-  """An issue that should be fixed.
-
-  It will not stop the feed from being successfully processed but may lead to
-  undefined errors.
-  """
-
-  description = "Warning"
-
-
-class ElectionInfo(ElectionException):
-  """Information that user needs to know about following XML best practices."""
-
-  description = "Info"
-
-
-class ErrorLogEntry(object):
-  line = None
-  message = None
-
-  def __init__(self, line, message):
-    self.line = line
-    self.message = message
-
-
 class SchemaHandler(object):
   """Base class for anything that parses an XML schema document."""
   _XSCHEMA_NAMESPACE = "http://www.w3.org/2001/XMLSchema"
@@ -125,11 +77,14 @@ class BaseRule(SchemaHandler):
   def set_option(self, option):
     """Used to set commandline options for the rule.
 
-    The rule must have the option_name attribute, otherwise raise an
-    exception.
+    Args:
+      option: commandline option object.
+
+    Raises:
+      ElectionException: the rule must have the option_name attribute.
     """
     if not hasattr(self, option.option_name):
-      raise ElectionException("Invalid attribute set")
+      raise loggers.ElectionException("Invalid attribute set")
     setattr(self, option.option_name, option.option_value)
 
   def setup(self):
@@ -146,6 +101,90 @@ class TreeRule(BaseRule):
     """Checks entire tree."""
 
 
+class ValidReferenceRule(TreeRule):
+  """Rule that makes sure reference values are properly defined."""
+
+  def __init__(self, election_tree, schema_file, missing_element="data"):
+    super(ValidReferenceRule, self).__init__(election_tree, schema_file)
+    self.missing_element = missing_element
+
+  def _gather_reference_values(self):
+    """Collect a set of all values that are referencing a pre-defined value.
+
+    Ex: A party leader ID should reference an ID from a PersonCollection.
+    This method should return a set of all party leader IDs.
+    """
+    raise NotImplementedError
+
+  def _gather_defined_values(self):
+    """Collect a set of the pre-defined values that are being referenced.
+
+    Ex: A party leader ID should reference an ID from a PersonCollection.
+    This method should return a set of all PersonIDs from the PersonCollection.
+    """
+    raise NotImplementedError
+
+  def check(self):
+    reference_ids = self._gather_reference_values()
+    defined_ids = self._gather_defined_values()
+    invalid_references = reference_ids - defined_ids
+
+    if invalid_references:
+      raise loggers.ElectionError("No defined {} for {} found in the feed."
+                                  .format(self.missing_element,
+                                          ", ".join(invalid_references)))
+
+
+class DateRule(BaseRule):
+  """Base rule used for date validations.
+
+  When validating dates, this rule can be used to gather start and
+  end date values.
+  """
+
+  def elements(self):
+    return ["Election"]
+
+  def gather_dates(self, election_element):
+    """Gather StartDate and EndDate values for the provided element.
+
+    An election element should have a start and end date in the desired format.
+    These dates should be extracted and set as instance variables to be used
+    in validation checks.
+
+    Args:
+      election_element: An election element.
+
+    Raises:
+      ElectionError: dates need to be properly formatted.
+    """
+    error_log = []
+    self.today = datetime.datetime.now().date()
+
+    self.start_elem = election_element.find("StartDate")
+    try:
+      self.start_date = datetime.datetime.strptime(
+          self.start_elem.text, "%Y-%m-%d").date()
+    except ValueError:
+      error_message = "The StartDate text should be of the format yyyy-mm-dd"
+      error_log.append(loggers.ErrorLogEntry(
+          self.start_elem.sourceline, error_message))
+
+    self.end_elem = election_element.find("EndDate")
+    try:
+      self.end_date = datetime.datetime.strptime(
+          self.end_elem.text, "%Y-%m-%d").date()
+    except ValueError:
+      error_message = "The EndDate text should be of the format yyyy-mm-dd"
+      error_log.append(loggers.ErrorLogEntry(
+          self.end_elem.sourceline, error_message))
+
+    if error_log:
+      raise loggers.ElectionError(
+          "The format for the election dates are invalid: ",
+          error_log)
+
+
 class RuleOption(object):
   class_name = None
   option_name = None
@@ -159,7 +198,12 @@ class RuleOption(object):
 class RulesRegistry(SchemaHandler):
   """Registry of rules and the elements they check."""
 
-  _SEVERITIES = (ElectionInfo, ElectionWarning, ElectionError)
+  _SEVERITIES = (loggers.ElectionInfo,
+                 loggers.ElectionWarning,
+                 loggers.ElectionError)
+
+  _TOP_LEVEL_ENTITIES = set(
+      ["Party", "GpUnit", "Office", "Person", "Candidate", "Contest"])
 
   def __init__(self, election_file, schema_file, rule_classes_to_check,
                rule_options):
@@ -172,23 +216,21 @@ class RulesRegistry(SchemaHandler):
     self.exception_counts = {}
     self.exception_rule_counts = {}
     self.total_count = 0
+    self.election_tree = None
 
     for e_type in self._SEVERITIES:
       self.exceptions[e_type] = dict()
       self.exception_counts[e_type] = 0
       self.exception_rule_counts[e_type] = dict()
 
-  def register_rules(self, election_tree):
+  def register_rules(self):
     """Register all the rules to be checked.
 
     Returns:
       A dictionary of elements and rules that check each element
-
-    Args:
-      election_tree: election tree to be checked
     """
     for rule in self.rule_classes_to_check:
-      rule_instance = rule(election_tree, self.schema_file)
+      rule_instance = rule(self.election_tree, self.schema_file)
       if rule.__name__ in self.rule_options.keys():
         for option in self.rule_options[rule.__name__]:
           rule_instance.set_option(option)
@@ -200,7 +242,8 @@ class RulesRegistry(SchemaHandler):
           self.registry[element] = [rule_instance]
 
   def exception_handler(self, rule, exception):
-    for e_type in self.exceptions.keys():
+    """Gather error counts by type, class, and total."""
+    for e_type in self.exceptions:
       if issubclass(exception.__class__, e_type):
         if rule.__class__ not in self.exceptions[e_type]:
           self.exceptions[e_type][rule.__class__] = []
@@ -232,6 +275,7 @@ class RulesRegistry(SchemaHandler):
       e_type_name = e_type.description
       print("{0:6d} {1} message{2} found".format(self.exception_counts[e_type],
                                                  e_type_name, suffix))
+      # pylint: disable=cell-var-from-loop
       # Within the error severity, sort from most common to least common.
       for rule_class in sorted(
           self.exceptions[e_type].keys(),
@@ -246,37 +290,58 @@ class RulesRegistry(SchemaHandler):
                                                   e_type_name, rule_suffix))
         if verbose:
           for exception in self.exceptions[e_type][rule_class]:
-            if exception.error_log:
-              for error in exception.error_log:
-                if error.line is not None:
-                  print(" " * 14 +
-                        "Line {0}: {1}".format(error.line, error.message))
-                else:
-                  print(" " * 14 + "{0}".format(error.message))
-            else:
+            if not exception.error_log:
               print(" " * 14 + "{0}".format(exception))
+              continue
+            for error in exception.error_log:
+              if error.line is not None:
+                print(" " * 14 +
+                      u"Line {0}: {1}".format(error.line, error.message))
+              else:
+                print(" " * 14 + u"{0}".format(error.message))
+
+  def count_stats(self):
+    """Aggregates the counts for each top level entity."""
+    if self.election_tree:
+      # Find the top-level entities.
+      entity_path_str = ".//{0}Collection//{1}"
+      print("\n" + " " * 5 + "Entity and Attribute Counts:")
+      for entity_name in stats.ENTITY_STATS:
+        entity_instances = self.election_tree.findall(
+            entity_path_str.format(entity_name, entity_name))
+        if entity_instances:
+          # If top-level entity exists, instantiate a stat counter with total.
+          entity_stats = stats.ENTITY_STATS[entity_name](len(entity_instances))
+          # Then for each possible nested attribute, add count for those.
+          for attr in entity_stats.attribute_counts:
+            for instance in entity_instances:
+              entity_stats.increment_attribute(
+                  attr, len(instance.findall(".//{}".format(attr))))
+          print(entity_stats)
 
   def check_rules(self):
     """Checks all rules."""
     try:
-      election_tree = etree.parse(self.election_file)
+      self.election_tree = etree.parse(self.election_file)
     except etree.LxmlError as e:
       print("Fatal Error. XML file could not be parsed. {}".format(e))
-      self.exception_counts[ElectionError] += 1
+      self.exception_counts[loggers.ElectionError] += 1
       self.total_count += 1
       return
-    self.register_rules(election_tree)
+    self.register_rules()
     for rule in self.registry.get("tree", []):
       try:
         rule.check()
-      except ElectionException as e:
+      except loggers.ElectionException as e:
         self.exception_handler(rule, e)
-    for event, element in etree.iterwalk(election_tree, events=("end",)):
+    for _, element in etree.iterwalk(self.election_tree, events=("end",)):
       tag = self.get_element_class(element)
-      if not tag or tag not in self.registry.keys():
+
+      if not tag or tag not in self.registry:
         continue
+
       for element_rule in self.registry[tag]:
         try:
           element_rule.check(element)
-        except ElectionException as e:
+        except loggers.ElectionException as e:
           self.exception_handler(element_rule, e)
