@@ -11,7 +11,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 """NIST CDF validation library.
 
 A set of rules that can be used to validate a NIST 1500-100 file containing
@@ -25,12 +24,15 @@ from __future__ import print_function
 import argparse
 import codecs
 import os
+import re
 
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import hashes
 from election_results_xml_validator import base
+from election_results_xml_validator import loggers
 from election_results_xml_validator import rules
 from election_results_xml_validator import version
+import github
 
 
 def _validate_path(parser, arg):
@@ -58,26 +60,44 @@ def _validate_rules(parser, arg):
 def _validate_severity(parser, arg):
   """Check that the severity level provided is correct."""
 
-  _VALID_SEVERITIES = {"info": 0, "warning": 1, "error": 2}
-  if arg.strip().lower() not in _VALID_SEVERITIES:
+  valid_severities = {"info": 0, "warning": 1, "error": 2}
+  if arg.strip().lower() not in valid_severities:
     parser.error("Invalid severity. Options are error, warning, or info")
   else:
-    return _VALID_SEVERITIES[arg.strip().lower()]
+    return valid_severities[arg.strip().lower()]
 
 
+# pylint: disable=g-doc-args
+# pylint: disable=g-doc-return-or-yield
 def _validate_country_codes(parser, arg):
   """Check that the supplied 2 country code is correct.
 
   The repo is at https://github.com/opencivicdata/ocd-division-ids
   """
-  country_codes = [
-      "au", "ca", "cl", "de", "fi", "in", "nz", "mx", "ua", "us", "br"
-  ]
-  if arg.strip().lower() not in country_codes:
-    parser.error("Invalid country code. Available codes are: %s" %
-                 ", ".join(country_codes))
-  else:
-    return arg.strip().lower()
+  country_code = arg.strip().lower()
+
+  # 'us' is the default country code and will always be valid.
+  # This is so we bypass the call to the GitHub API when no -c flag
+  if country_code == "us":
+    return country_code
+
+  github_api = github.Github()
+  country_ids = github_api.get_repo(
+      "opencivicdata/ocd-division-ids").get_contents("identifiers")
+  valid_codes = []
+
+  for content_file in country_ids:
+    if content_file.type == "file":
+      result = re.search(r"country-([a-z]{2})\.csv", content_file.name)
+      if result:
+        ocd_id = result.group(1)
+        if country_code == ocd_id:
+          return country_code
+        else:
+          valid_codes.append(ocd_id)
+
+  parser.error("Invalid country code. Available codes are: %s" %
+               ", ".join(valid_codes))
 
 
 def arg_parser():
@@ -237,11 +257,11 @@ def main():
     errors = []
 
     for election_file in xml_files:
-      print("\n--------- Results after validating file: {0} "
-            .format(election_file))
+      print("\n--------- Results after validating file: {0} ".format(
+          election_file))
 
-      if (not election_file.endswith(".xml")
-          or not os.stat(election_file).st_size):
+      if (not election_file.endswith(".xml") or
+          not os.stat(election_file).st_size):
         print("{0} is not a valid XML file.".format(election_file))
         errors.append(3)
         continue
@@ -254,15 +274,18 @@ def main():
           rule_options=rule_options)
       registry.check_rules()
       registry.print_exceptions(options.severity, options.verbose)
-      if registry.exception_counts[base.ElectionError]:
+      if options.verbose:
+        registry.count_stats()
+      if registry.exception_counts[loggers.ElectionError]:
         errors.append(3)
-      elif registry.exception_counts[base.ElectionWarning]:
+      elif registry.exception_counts[loggers.ElectionWarning]:
         errors.append(2)
-      elif registry.exception_counts[base.ElectionInfo]:
+      elif registry.exception_counts[loggers.ElectionInfo]:
         errors.append(1)
       else:
         errors.append(0)
     return max(errors)
 
+
 if __name__ == "__main__":
   main()
